package ab.wifisdab;

/**
 * @author Andreas Butti
 * 
 * @license GPL
 */

import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.IOException;

import ab.wifisdab.control.ImageDownloader;
import ab.wifisdab.control.ImageDownloaderListener;
import ab.wifisdab.control.NotificationProgressController;
import android.app.Activity;
import android.app.NotificationManager;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.graphics.drawable.Drawable;
import android.os.Bundle;
import android.os.Environment;
import android.preference.PreferenceManager;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.CheckBox;
import android.widget.ImageView;
import android.widget.TextView;
import butt.wifisdaccess.WiFiSdAccess;
import butt.wifisdaccess.WiFiSdListener;

/**
 * Main Application interface
 * 
 * @author Andreas Butti
 */
public class MainActivity extends Activity {

	/**
	 * Download Queue for Images
	 */
	private ImageDownloader imageDownloader = new ImageDownloader();

	/**
	 * Progress of image download
	 */
	private NotificationProgressController notification;

	private WiFiSdListener listener = new WiFiSdListener() {

		@Override
		public void connectionClosed() {
			runOnUiThread(new Runnable() {
				public void run() {
					log(getString(R.string.main_connection_lost));
				}
			});
		}

		@Override
		public void newImage(final String path) {
			runOnUiThread(new Runnable() {
				public void run() {
					CheckBox cbDownload = (CheckBox) findViewById(R.id.cbAutodownloadNewImages);

					if (cbDownload.isChecked()) {
						log(getString(R.string.main_got_new_image_download, path));

						imageDownloader.add(path);
						notification.addWaiting(path);

					} else {
						log(getString(R.string.main_got_new_image_download_disabled, path));
					}
				}
			});

		}

		@Override
		public void errorOccured(Exception ex) {
		}

		@Override
		public void connectionEstablished() {
		}

	};

	private ImageDownloaderListener downloadListener = new ImageDownloaderListener() {

		@Override
		public void gotPreview(final byte[] preview, String path) {
			final Drawable drawable = Drawable.createFromStream(new ByteArrayInputStream(preview), "/remoteSD/" + path);

			runOnUiThread(new Runnable() {
				public void run() {
					ImageView imageView = (ImageView) findViewById(R.id.imagePreview);
					imageView.setImageDrawable(drawable);
				}
			});
		}

		@Override
		public void downloadFinished(final String path, String androidPath, int seconds, int bytesPerSeconds) {
			notification.imageFinished(path, seconds, bytesPerSeconds);

			if (seconds < 0) {
				// error
				runOnUiThread(new Runnable() {
					public void run() {
						log(getString(R.string.notification_finished_error, path));
					}
				});
			} else {

				// out of memory
//				final Drawable drawable = Drawable.createFromPath(androidPath);
//
//				runOnUiThread(new Runnable() {
//					public void run() {
//						ImageView imageView = (ImageView) findViewById(R.id.imagePreview);
//						imageView.setImageDrawable(drawable);
//					}
//				});
			}
		}

		@Override
		public void downloadProgress(String path, int current, int amount) {
			notification.imageProgress(path, current, amount);
		}

	};

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.activity_main);

		NotificationManager notifyManager = (NotificationManager) getSystemService(Context.NOTIFICATION_SERVICE);
		notification = new NotificationProgressController(notifyManager, this);

		getSd().addListener(listener);

		initSettings();
		initImageDownloader();
	}

	/**
	 * Init the downloader
	 */
	private void initImageDownloader() {
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
		prefs.registerOnSharedPreferenceChangeListener(imageDownloader);
		String path = prefs.getString("pref_key_save_location", "");

		imageDownloader.setImageSavePath(path);
		imageDownloader.setSd(getSd());
		imageDownloader.addListener(downloadListener);
		imageDownloader.start();
	}

	/**
	 * Initialize settings with values
	 */
	private void initSettings() {
		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
		String path = prefs.getString("pref_key_save_location", "");

		File file = new File(path);
		if (path.isEmpty() || !file.canWrite()) {
			path = Environment.getExternalStorageDirectory().getPath() + "/WiFiSdImages";
			Editor editor = prefs.edit();
			editor.putString("pref_key_save_location", path);
			editor.apply();
		}
	}

	private WiFiSdAccess getSd() {
		return ((WiFiSdApplication) getApplication()).getSd();
	}

	@Override
	protected void onDestroy() {
		getSd().removeListener(listener);
		try {
			getSd().close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		SharedPreferences prefs = PreferenceManager.getDefaultSharedPreferences(this);
		prefs.unregisterOnSharedPreferenceChangeListener(imageDownloader);

		imageDownloader.stop();
		imageDownloader.removeListener(downloadListener);

		super.onDestroy();
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle item selection
		switch (item.getItemId()) {
		case R.id.action_settings:
			startActivity(new Intent(this, SettingsActivity.class));
			return true;
		default:
			return super.onOptionsItemSelected(item);
		}
	}

	public void onConnect(View view) {
		startActivity(new Intent(this, ConnectActivity.class));
	}

	/**
	 * Logs a text message
	 * 
	 * @param message
	 *            The Message
	 */
	private void log(String message) {
		TextView text = (TextView) findViewById(R.id.textState);
		text.setText(message);
	}

	/**
	 * Logs a text message
	 * 
	 * @param message
	 *            The Message
	 * @param ex
	 *            Exception
	 */
	private void log(String message, Exception ex) {
		TextView text = (TextView) findViewById(R.id.textState);
		text.setText(message);

		ex.printStackTrace();
	}

	public void onDisconnect(View view) {
		try {
			getSd().close();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

}
