package ab.wifisdab.control;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.LinkedList;

import android.content.SharedPreferences;
import android.content.SharedPreferences.OnSharedPreferenceChangeListener;
import butt.wifisdaccess.DownloadProgressListener;
import butt.wifisdaccess.WiFiSdAccess;
import butt.wifisdaccess.WiFiSdException;

/**
 * Image Download Helper class
 * 
 * @author Andreas Butti
 * @license GPL
 */
public class ImageDownloader implements OnSharedPreferenceChangeListener {
	/**
	 * Image Queueu
	 */
	private LinkedList<String> downloadQueue = new LinkedList<String>();

	/**
	 * synchronisation obejct
	 */
	private Object synchronisation = new Object();

	/**
	 * Path to save the images to, e.g. /mnt/sdcard/WifiDownload etc.
	 */
	private String imageSavePath = "";

	/**
	 * The downloading thread
	 */
	private Thread threads;

	/**
	 * Flag to stop the thread
	 */
	private boolean threadRunning = true;

	/**
	 * Sd Card access
	 */
	private WiFiSdAccess sd;

	/**
	 * Listeners
	 */
	private ArrayList<ImageDownloaderListener> listener = new ArrayList<ImageDownloaderListener>();

	/**
	 * Adds a new image to the download queue
	 * 
	 * @param path
	 *            the path of the image
	 */
	public void add(String path) {
		synchronized (downloadQueue) {
			downloadQueue.addFirst(path);
		}

		synchronized (synchronisation) {
			synchronisation.notifyAll();
		}
	}

	/**
	 * Start the downloader
	 */
	public void start() {
		threadRunning = true;
		threads = new Thread(new Runnable() {

			@Override
			public void run() {
				while (threadRunning) {
					String path = null;
					synchronized (downloadQueue) {
						if (downloadQueue.size() > 0) {
							path = downloadQueue.removeLast();
						}
					}

					if (path != null) {
						downloadPreview(path);
						downloadImage(path);
					}

					if (path == null) {
						// wait for next image
						synchronized (synchronisation) {
							try {
								synchronisation.wait();
							} catch (InterruptedException e) {
							}
						}
					}
				}
			}

		});
		threads.start();
	}

	/**
	 * Downloads the preview image and shows them
	 * 
	 * @param path
	 *            the path
	 */
	private void downloadPreview(String path) {
		byte[] preview = sd.downloadPreview(path);

		synchronized (this.listener) {
			for (ImageDownloaderListener l : this.listener) {
				l.gotPreview(preview, path);
			}
		}
	}

	/**
	 * Fire download state of an image to listener
	 * 
	 * @param path
	 *            The path on the SD Card
	 * @param current
	 *            Downloaded bytes
	 * @param amount
	 *            total bytes
	 */
	private void fireDownloadState(String path, int current, int amount) {
		synchronized (this.listener) {
			for (ImageDownloaderListener l : this.listener) {
				l.downloadProgress(path, current, amount);
			}
		}
	}

	/**
	 * Fire download finished message to listener
	 * 
	 * @param path
	 *            The path on the SD Card
	 * @param androidPath
	 *            The local path on the Android device
	 * @param seconds
	 *            Complete seconds
	 * @param bytesPerSeconds
	 *            Speed
	 */
	private void fireDownloadFinished(String path, String androidPath, int seconds, int bytesPerSeconds) {
		synchronized (this.listener) {
			for (ImageDownloaderListener l : this.listener) {
				l.downloadFinished(path, androidPath, seconds, bytesPerSeconds);
			}
		}
	}

	/**
	 * Downloads the image to the Android device SD Card
	 * 
	 * @param path
	 *            the path
	 */
	private void downloadImage(final String path) {
		// "/mnt/sd/DCIM/199_WIFI/IMG_9450.JPG"

		final String androidPath;
		// TODO remove /mnt/sd on Transcend implementation
		if (path.startsWith("/mnt/sd/")) {
			androidPath = getImageSavePath() + "/" + path.substring(8);
		} else {
			androidPath = getImageSavePath() + "/" + path;
		}

		try {

			File folder = new File(androidPath).getParentFile();
			if (!folder.exists() && !folder.mkdirs()) {
				// TODO Popup message
				System.err.println("Could not create folder: " + folder.getAbsolutePath());
				
				fireDownloadFinished(path, androidPath, -1, -1);
				return;
			}

			OutputStream out = new FileOutputStream(androidPath);

			sd.downloadFile(path, out, new DownloadProgressListener() {

				@Override
				public void progress(int current, int amount) {
					fireDownloadState(path, current, amount);
				}

				@Override
				public void finished(int seconds, int bytesPerSeconds) {
					fireDownloadFinished(path, androidPath, seconds, bytesPerSeconds);
				}
			});

			out.close();

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (WiFiSdException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Stops the downloader
	 */
	public void stop() {
		threadRunning = false;
		synchronized (synchronisation) {
			synchronisation.notifyAll();
		}
		try {
			threads.join();
		} catch (InterruptedException e) {
		}
	}

	@Override
	public void onSharedPreferenceChanged(SharedPreferences sharedPreferences, String key) {
		if ("pref_key_save_location".equals(key)) {
			setImageSavePath(sharedPreferences.getString("pref_key_save_location", ""));
		}
	}

	public void setImageSavePath(String imageSavePath) {
		this.imageSavePath = imageSavePath;
	}

	public String getImageSavePath() {
		return imageSavePath;
	}

	public void setSd(WiFiSdAccess sd) {
		this.sd = sd;
	}

	public void addListener(ImageDownloaderListener listener) {
		synchronized (this.listener) {
			this.listener.add(listener);
		}
	}

	public void removeListener(ImageDownloaderListener listener) {
		synchronized (this.listener) {
			this.listener.remove(listener);
		}
	}

}
