package ab.wifisdab.control;

import java.util.HashMap;

import ab.wifisdab.R;
import android.app.Notification;
import android.app.NotificationManager;
import android.content.Context;
import android.support.v4.app.NotificationCompat;
import android.support.v4.app.NotificationCompat.Builder;
import android.util.Log;

/**
 * Controller for notifications
 * 
 * @author Andreas Butti
 * 
 * @license GPL
 */
public class NotificationProgressController {

	/**
	 * NotificationManager
	 */
	private NotificationManager notifyManager;

	/**
	 * Context to create new notifications
	 */
	private Context context;

	/**
	 * Currently active notifications
	 */
	private HashMap<String, NotificationContainer> notifications = new HashMap<String, NotificationContainer>();

	private static class NotificationContainer {
		private Builder builder;
		private int id;
		private static int nextId = 1;

		private String waiting;
		private String downloading;
		private String finished;
		private String downloadFailed;

		private int percent = -1;

		public NotificationContainer(Builder builder, String waiting, String downloading, String finished,
				String downloadFailed) {
			this.builder = builder;

			this.waiting = waiting;
			this.downloading = downloading;
			this.finished = finished;
			this.downloadFailed = downloadFailed;

			synchronized (NotificationContainer.class) {
				id = nextId++;
			}
		}

		public int getId() {
			return id;
		}

		public void setTextWaiting() {
			builder.setContentText(waiting);
			// Removes the progress bar
			builder.setProgress(0, 0, true);

			percent = -1;
		}

		public void setTextDownloading(int percent) {
			this.percent = percent;
			builder.setContentText(downloading);
			// Add progress bar
			builder.setProgress(100, percent, false);
		}

		public void setTextFinished(int seconds, int bytesPerSeconds) {
			if (seconds < 0) {
				// download failed
				builder.setContentText(downloadFailed);
			} else {
				double ks = bytesPerSeconds /= 1024.0;
				builder.setContentText(finished + " (" + seconds + "s, " + ks + "kB/s)");
			}

			// Removes the progress bar
			builder.setProgress(0, 0, false);

			percent = -1;
		}

		public Notification build() {
			return builder.build();
		}

	}

	public NotificationProgressController(NotificationManager notifyManager, Context context) {
		this.notifyManager = notifyManager;
		this.context = context;
	}

	private synchronized NotificationContainer getNotificationForPath(String path) {
		NotificationContainer notificationContainer = notifications.get(path);
		if (notificationContainer == null) {
			Builder notification = new NotificationCompat.Builder(context);
			notification.setContentTitle(context.getString(R.string.notification_title)).setSmallIcon(
					R.drawable.ic_launcher);

			notificationContainer = new NotificationContainer(notification, context.getString(
					R.string.notification_waiting_for, path),
					context.getString(R.string.notification_downloading, path), context.getString(
							R.string.notification_finished, path), context.getString(
							R.string.notification_finished_error, path));

			notifications.put(path, notificationContainer);
		}
		return notificationContainer;
	}

	/**
	 * The download of the files was finished
	 * 
	 * @param path
	 *            The path on the SD Card
	 * @param seconds
	 *            Complete seconds
	 * @param bytesPerSeconds
	 *            Speed
	 */
	public void imageFinished(String path, int seconds, int bytesPerSeconds) {
		NotificationContainer notification = getNotificationForPath(path);
		notification.setTextFinished(seconds, bytesPerSeconds);
		notifyManager.notify(notification.getId(), notification.build());
	}

	private static final String TAG = "WiFiSd";

	/**
	 * Fire download state of an image has increased
	 * 
	 * @param path
	 *            The path on the SD Card
	 * @param current
	 *            Downloaded bytes
	 * @param amount
	 *            total bytes
	 */
	public void imageProgress(String path, int current, int amount) {
		NotificationContainer notification = getNotificationForPath(path);

		int percent = (int) (current * 100.0 / amount);

		if (notification.percent == percent) {
			Log.v(TAG, "ignore");
			return;
		} else {
			Log.v(TAG, "percent = " + percent);
		}

		notification.setTextDownloading(percent);
		notifyManager.notify(notification.getId(), notification.build());
	}

	public void addWaiting(String path) {
		NotificationContainer notification = getNotificationForPath(path);

		notification.setTextWaiting();
		notifyManager.notify(notification.getId(), notification.build());
	}

}
