package butt.wifisdaccess.transcend;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.UnsupportedEncodingException;
import java.net.MalformedURLException;
import java.net.Socket;
import java.net.URL;
import java.net.URLEncoder;
import java.net.UnknownHostException;
import java.nio.charset.Charset;
import java.text.MessageFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import butt.wifisdaccess.DownloadProgressListener;
import butt.wifisdaccess.InvalidConfigurationException;
import butt.wifisdaccess.WiFiSdException;
import butt.wifisdaccess.base.WiFiSdAccessBase;
import butt.wifisdaccess.model.Exif;
import butt.wifisdaccess.model.Size;
import butt.wifisdaccess.util.HttpUtil;

/**
 * Transcend WiFi SD Access Implementation
 * 
 * @author Andreas Butti
 * 
 */
public class WiFiSdAccessTranscend extends WiFiSdAccessBase {
	/**
	 * Control communication port
	 */
	public static final String PROP_CONTROL_PORT = "control.port";

	/**
	 * File information URL
	 */
	public static final String PROP_URL_FILE_INFO = "url.file.info";

	/**
	 * File exif URL
	 */
	public static final String PROP_URL_FILE_EXIF = "url.file.exif";

	/**
	 * File download URL
	 */
	public static final String PROP_URL_FILE_DOWNLOAD = "url.file.download";

	/**
	 * File preview URL
	 */
	public static final String PROP_URL_FILE_PREVIEW = "url.file.preview";

	/**
	 * Thread for control communication
	 */
	private Thread controlThread;

	/**
	 * Socket for control communication
	 */
	private Socket controlSocket;

	/**
	 * The base url of the card (e.g. 192.168.11.254)
	 */
	private String connectionUrl;

	/**
	 * EXIF date format
	 * 
	 * 2013:08:03 11:57:47
	 */
	private static final SimpleDateFormat dateFormat = new SimpleDateFormat(
			"yyyy:MM:dd HH:mm:ss");

	/**
	 * Logger
	 */
	private final static Logger LOGGER = Logger
			.getLogger(WiFiSdAccessTranscend.class.getName());

	/**
	 * Constructor
	 */
	public WiFiSdAccessTranscend() {
		setProperty(PROP_CONTROL_PORT, "5566");
		// e.g.
		// /cgi-bin/tscmd?CMD=GET_EXIF&PIC=%2Fwww%2Fsd%2FDCIM%2F100CANON%2FIMG_9380.JPG

		setProperty(PROP_URL_FILE_EXIF, "/cgi-bin/tscmd?CMD=GET_EXIF&PIC={0}");
		// e.g.
		// TODO example!
		setProperty(PROP_URL_FILE_INFO,
				"/cgi-bin/tscmd?CMD=GET_FILE_INFO&FILE={0}");
		// e.g.
		// /cgi-bin/wifi_download?fn=IMG_9380.JPG&fd=%2Fwww%2Fsd%2FDCIM%2F100CANON
		setProperty(PROP_URL_FILE_DOWNLOAD,
				"/cgi-bin/wifi_download?fn={0}&fd={1}");
		// e.g.
		// /cgi-bin/thumbNail?fn=/www/sd/DCIM/100CANON/IMG_9380.JPG
		setProperty(PROP_URL_FILE_PREVIEW, "/cgi-bin/thumbNail?fn={0}");
	}

	@Override
	public void connect(final String url) {
		this.connectionUrl = url;

		controlThread = new Thread(new Runnable() {

			@Override
			public void run() {
				LOGGER.log(Level.FINE, "Control thread started");

				networkingThread();

				LOGGER.log(Level.FINE, "Control thread ended");
			}
		});

		controlThread.start();
	}

	protected void networkingThread() {
		int port = 5566;
		try {
			port = Integer.parseInt(getProperty(PROP_CONTROL_PORT));
		} catch (NumberFormatException ex) {
			fireError(new InvalidConfigurationException(PROP_CONTROL_PORT
					+ " is not an integer!", ex));
			return;
		}

		LOGGER.log(Level.FINE, "Connect to " + this.connectionUrl + ":" + port);

		try {
			controlSocket = new Socket(this.connectionUrl, port);
		} catch (UnknownHostException e) {
			fireError(new WiFiSdException("could not connect to card", e));
			return;
		} catch (IOException e) {
			fireError(new WiFiSdException("could not connect to card", e));
			return;
		}

		fireConnectionEstablished();
		
		final Charset charset = Charset.forName("ISO-8859-1");

		InputStream in;
		try {
			in = controlSocket.getInputStream();
		} catch (IOException e) {
			fireConnectionClosed();

			LOGGER.log(Level.SEVERE,
					"Could not read input stream from control thread", e);

			return;
		}

		byte[] buffer = new byte[1024]; // 1kByte

		while (controlSocket.isConnected()) {
			try {
				int readcount = in.read(buffer);

				// need a valid path and start / stop
				if (readcount < 5) {
					LOGGER.log(Level.INFO, "Got unknown message: "
							+ new String(buffer, 0, readcount));
					continue;
				}

				if (buffer[0] != '>') {
					LOGGER.log(Level.INFO, "Got unknown message: "
							+ new String(buffer, 0, readcount));
					continue;
				}

				if (buffer[readcount - 1] != 0) {
					LOGGER.log(Level.INFO, "Got unknown message: "
							+ new String(buffer, 0, readcount));
					continue;
				}

				fireNewImage(new String(buffer, 1, readcount - 2, charset));

			} catch (IOException e) {
				fireConnectionClosed();

				LOGGER.log(Level.SEVERE,
						"Could not read input stream from control thread", e);
				return;
			}

		}

		fireConnectionClosed();
	}

	@Override
	public void close() throws IOException {
		controlSocket.close();
		try {
			controlThread.wait();
		} catch (InterruptedException e) {
			throw new IOException(e);
		}
	}

	/**
	 * Base URL for all connections
	 */
	private String getBasurl() {
		return "http://" + connectionUrl;
	}

	/**
	 * Parses a colon separated list
	 * 
	 * @param data
	 *            Data String
	 * @return List
	 */
	private HashMap<String, String> parseList(String data) {
		HashMap<String, String> list = new HashMap<String, String>();
		for (String line : data.split("(<br>?\n)")) {
			line = line.trim();

			int colon = line.indexOf(':');
			if (colon < 0) {
				continue;
			}

			list.put(line.substring(0, colon).trim(), line.substring(colon + 1)
					.trim());

		}
		return list;
	}

	public static long parseFilesize(String in) {
		in = in.trim();
		in = in.replaceAll(",", ".");
		try {
			return Long.parseLong(in);
		} catch (NumberFormatException e) {
		}
		final Matcher m = Pattern.compile("([\\d.,]+)\\s*(\\w)").matcher(in);
		m.find();
		int scale = 1;
		switch (m.group(2).charAt(0)) {
		case 'G':
			scale *= 1024;
		case 'M':
			scale *= 1024;
		case 'K':
			break;
		default:
			throw new IllegalArgumentException();
		}
		return Math.round(Double.parseDouble(m.group(1)) * scale);
	}

	@Override
	public byte[] downloadPreview(String path) {
		try {
			String encodedPath = URLEncoder.encode(path, "ISO-8859-1");
			URL url = new URL(getBasurl()
					+ MessageFormat.format(getProperty(PROP_URL_FILE_PREVIEW),
							encodedPath));

			return HttpUtil.getHttpSimple(url);
		} catch (MalformedURLException e) {
			LOGGER.log(Level.WARNING, "Could not download preview: " + path, e);
			return null;
		} catch (IOException e) {
			LOGGER.log(Level.WARNING, "Could not download preview: " + path, e);
			return null;
		}
	}

	@Override
	public void downloadFile(String path, OutputStream out,
			DownloadProgressListener listener) throws WiFiSdException {
		try {
			int pos = path.lastIndexOf('/');
			if (pos < 0) {
				throw new WiFiSdException("path contains no slash!: " + path);
			}

			String encodedName = URLEncoder.encode(path.substring(pos + 1),
					"ISO-8859-1");
			String encodedPath = URLEncoder.encode(path.substring(0, pos),
					"ISO-8859-1");
			URL url = new URL(getBasurl()
					+ MessageFormat.format(getProperty(PROP_URL_FILE_DOWNLOAD),
							encodedName, encodedPath));

			HttpUtil.download(url, out, listener);

		} catch (MalformedURLException e) {
			e.printStackTrace();
			throw new WiFiSdException("Error Downloading file", e);
		} catch (IOException e) {
			e.printStackTrace();
			throw new WiFiSdException("Error Downloading file", e);
		}
	}

	@Override
	public Exif getExifFor(String path) {
		try {
			String encodedPath = URLEncoder.encode(path, "ISO-8859-1");
			URL url = new URL(getBasurl()
					+ MessageFormat.format(getProperty(PROP_URL_FILE_EXIF),
							encodedPath));

			byte[] data = HttpUtil.getHttpSimple(url);

			HashMap<String, String> list = parseList(new String(data));

			// Fail: Cmd GET_EXIF ret = -1

			if (list.containsKey("Fail")) {
				LOGGER.log(Level.WARNING, "Get exif for \"" + path
						+ "\" failed with reason: " + list.get("Fail"));
				return null;
			}

			if (!list.containsKey("Success")) {
				LOGGER.log(Level.WARNING, "Get exif for \"" + path
						+ "\" failed");
				return null;
			}

			Exif exif = new Exif();

			String value = null;

			value = list.get("F"); // 35/10
			if (value != null) {
				try {
					if (value.contains("/")) {
						String[] tmp = value.split("/");
						exif.setAperture((double) Integer.parseInt(tmp[0]
								.trim()) / Integer.parseInt(tmp[1].trim()));
					} else {
						exif.setAperture(Integer.parseInt(value.trim()));
					}

				} catch (Exception e) {
					LOGGER.log(Level.WARNING, "Could not parse Aperture: "
							+ value);
				}
			}

			value = list.get("Filesize");
			if (value != null) {
				exif.setFileSize(parseFilesize(value));
			}

			value = list.get("Date");
			if (value != null) {
				// 2013:08:03 11:57:47
				try {
					Date date = dateFormat.parse(value);
					exif.setDate(date);
				} catch (ParseException e) {
					LOGGER.log(Level.WARNING, "Invalid Date value: " + value);
				}
			}

			value = list.get("file");
			if (value != null) {
				exif.setPath(value);
			}

			value = list.get("ISO");
			if (value != null) {
				try {
					exif.setIso(Integer.parseInt(value));
				} catch (NumberFormatException e) {
					LOGGER.log(Level.WARNING, "Invalid ISO value: " + value);
				}
			}

			value = list.get("EV");
			if (value != null) {
				exif.setEv(value);
			}

			value = list.get("Size");
			if (value != null) {
				try {
					String[] tmp = value.split("x");

					exif.setSize(new Size(Integer.parseInt(tmp[0].trim()),
							Integer.parseInt(tmp[1].trim())));
				} catch (Exception e) {
					LOGGER.log(Level.WARNING, "Could not parse dimension: "
							+ value);
				}

			}

			// exif>>> file: /mnt/sd/DCIM/100CANON/IMG_9390.JPG<br>
			// Size: 5184 x 3456<br>
			// Exposure: 1/40<br>
			// F: 35/10<br>
			// EV: -1/1<br>
			// ISO: 500<br>
			// Date: 2013:08:03 11:46:01<br>
			// Filesize: 4.7 MB<br>
			// Success: Cmd GET_EXIF

			return exif;
		} catch (UnsupportedEncodingException e) {
			LOGGER.log(Level.SEVERE, "Invalid encoding", e);
		} catch (MalformedURLException e) {
			LOGGER.log(Level.SEVERE, "Invalid URL", e);
		} catch (IOException e) {
			LOGGER.log(Level.SEVERE, "Connection error", e);
		}

		return null;
	}

}
